from keras.models import Sequential
from keras.layers import Dense
import numpy as np
from sklearn.model_selection import train_test_split
import matplotlib.pyplot as plt

# 2つのクラスの境界線
def f(x):
	return 2 * x * (x + 1) * (x - 1)

# データの作成(最終的に入力xとラベルtを得る)
data_num = 700 # データ数
x1 = np.random.rand(data_num) * 2 - 1 # [-1, 1)からランダムな値
x2 = np.random.rand(data_num) * 2 - 1 # [-1, 1)からランダムな値
x = np.c_[x1, x2] # x1とx2の各要素について、(x1i, x2i)と連結
t = x2 >= f(x1) # 各要素についてx2 >= f(x1)ならTrue(1)、そうでないならFalse(0)

# 訓練データとテストデータに分離(test_sizeはテストデータの割合)
x_train, x_test, t_train, t_test = train_test_split(x, t, test_size=0.2)

# 多層パーセプトロンの構築
model = Sequential() # 空のモデルを定義

model.add(Dense(5, activation="tanh", input_dim=2)) # 中間層(入力の次の層のみ入力データの次元を入力)
model.add(Dense(1, activation="sigmoid")) # 出力層

# モデルの各層やパラメータ数などの情報を出力
model.summary()

# 誤差関数などの設定
model.compile(loss="binary_crossentropy", optimizer="sgd", metrics=["accuracy"])

# 学習
history = model.fit(x_train, t_train, batch_size=1, epochs=50, validation_data=(x_test, t_test))

# 損失の履歴をプロット
plt.plot(history.history["loss"])
plt.plot(history.history["val_loss"])
plt.title("model loss")
plt.xlabel("epoch")
plt.ylabel("loss")
plt.legend(["loss", "val_loss"], loc="upper right")
plt.grid()
plt.savefig("loss.png")
plt.show()

# 精度の履歴をプロット
plt.plot(history.history["acc"])
plt.plot(history.history["val_acc"])
plt.title("model accuracy")
plt.xlabel("epoch")
plt.ylabel("accuracy")
plt.legend(["acc", "val_acc"], loc="lower right")
plt.grid()
plt.savefig("acc.png")
plt.show()

# テストデータを分類し描画
output = model.predict_classes(x_test) # 各入力に対しクラス番号を推定

# 推定した値で色を決めて描画
for i in range(x_test.shape[0]):
	if output[i] == 1:
		plt.plot(x_test[i, 0], x_test[i, 1], marker="o", color="r")
	else:
		plt.plot(x_test[i, 0], x_test[i, 1], marker="v", color="g")

# 正解の境界線の作成
x_fig = np.arange(-1, 1.1, 0.1)
y_fig = np.array(f(x_fig))

plt.plot(x_fig, y_fig) # 正解の境界線の描画
plt.show()