from keras.models import Sequential
from keras.layers import Dense, Dropout, Flatten
from keras.layers import Conv2D, MaxPooling2D
from keras.datasets import mnist
from keras.utils import np_utils
import matplotlib.pyplot as plt

# 多層パーセプトロンの構築
def build_multilayer_perceptron():
	model = Sequential()

	model.add(Dense(512, activation="relu", input_shape=(784,)))
	model.add(Dropout(0.5))
	model.add(Dense(512, activation="relu"))
	model.add(Dropout(0.5))
	model.add(Dense(10, activation="softmax"))

	return model

# CNNの構築
def build_cnn():
	model = Sequential()

	model.add(Conv2D(30, (5, 5), activation="relu", input_shape=(28, 28, 1)))
	model.add(MaxPooling2D(pool_size=(2, 2)))

	model.add(Flatten())
	model.add(Dense(100, activation="relu"))
	model.add(Dense(10, activation="softmax"))

	return model

# CNN(層の深いCNN)の構築
def build_deep_cnn():
	model = Sequential()

	model.add(Conv2D(16, (3, 3), padding="same", activation="relu", input_shape=(28, 28, 1)))
	model.add(Conv2D(16, (3, 3), padding="same", activation="relu"))
	model.add(MaxPooling2D(pool_size=(2, 2)))

	model.add(Conv2D(32, (3, 3), padding="same", activation="relu"))
	model.add(Conv2D(32, (3, 3), padding="same", activation="relu"))
	model.add(MaxPooling2D(pool_size=(2, 2)))

	model.add(Conv2D(64, (3, 3), padding="same", activation="relu"))
	model.add(Conv2D(64, (3, 3), padding="same", activation="relu"))
	model.add(MaxPooling2D(pool_size=(2, 2)))

	model.add(Flatten())
	model.add(Dense(50, activation="relu"))
	model.add(Dropout(0.5))
	model.add(Dense(10, activation="softmax"))

	return model

# MNISTデータの読み込み
(x_train, y_train), (x_test, y_test) = mnist.load_data()

# 多層パーセプロトンでの入力の形
x_train = x_train.reshape(60000, 784)
x_test = x_test.reshape(10000, 784)

# CNNでの入力の形
#x_train = x_train.reshape(60000, 28, 28, 1)
#x_test = x_test.reshape(10000, 28, 28, 1)

# numpy配列へ変換し入力値を[0,1]とする
x_train = x_train.astype("float32")
x_test = x_test.astype("float32")
x_train /= 255
x_test /= 255

# ラベルをone hot表現へ変換(例えば3は[0,0,1,0,0,0,0,0,0,0]になる)
y_train = np_utils.to_categorical(y_train, 10)
y_test = np_utils.to_categorical(y_test, 10)

# 使うモデルや誤差関数などの設定
model = build_multilayer_perceptron() # 試すモデルを設定
model.summary()
model.compile(loss="categorical_crossentropy", optimizer="adam", metrics=["accuracy"])

# 学習
history = model.fit(x_train, y_train, batch_size=128, epochs=20, validation_data=(x_test, y_test))

# 損失の履歴をプロット
plt.plot(history.history["loss"])
plt.plot(history.history["val_loss"])
plt.title("model loss")
plt.xlabel("epoch")
plt.ylabel("loss")
plt.legend(["loss", "val_loss"], loc="upper right")
plt.grid()
plt.savefig("loss.png")
plt.show()

# 精度の履歴をプロット
plt.plot(history.history["acc"])
plt.plot(history.history["val_acc"])
plt.title("model accuracy")
plt.xlabel("epoch")
plt.ylabel("accuracy")
plt.legend(["acc", "val_acc"], loc="lower right")
plt.grid()
plt.savefig("acc.png")
plt.show()